import sys
import tzlocal
import xmltodict
from datetime import datetime


def sizeof_fmt(num, suffix="B"):
    """Convert any number to human readable size. By Fred Cirera"""
    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
        if abs(num) < 1024.0:
            return "%3.1f %s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f %s%s" % (num, "Yi", suffix)


def check(response):
    """Check if response contains error result
    then raise exception."""
    sys.tracebacklimit = 0

    if "reason" in response:
        msg = (
            f"{response.get('reason')} [{response.get('status_code')}]\n"
            f"URL: {response.get('url')}"
        )
        raise ValueError(msg)


def check_plain(response):
    """Check if response contains error result
    then raise exception."""
    if response.text:
        response_dict = xmltodict.parse(response.text)

        if "Error" in response_dict:
            code = response_dict["Error"]["Code"]
            msg = response_dict["Error"]["Message"]
            raise ValueError(f"{code}: {msg}")


def human_date(unixtime):
    """Convert unix timestamp to human readable."""
    # divided by 1000 to convert from miliseconds to seconds
    unix_timestamp = int(unixtime) / 1000
    local_timezone = tzlocal.get_localzone()
    local_time = datetime.fromtimestamp(unix_timestamp, local_timezone)
    human_datetime = local_time.strftime("%Y-%m-%d %H:%M:%S%z (%Z)")
    return human_datetime



import os
import boto3
from cloudianapi.client import CloudianAPIClient
from requests_aws4auth import AWS4Auth

from obs.libs import config


def resource():
    """Take credential and create boto session

    :return: resource service client.
    """
    config.load_config_file()
    access_key = os.environ.get("OBS_USER_ACCESS_KEY")
    secret_key = os.environ.get("OBS_USER_SECRET_KEY")
    endpoint = os.environ.get("OBS_USER_URL")
    endpoint_url = f"https://{endpoint}"

    sess = boto3.Session(aws_access_key_id=access_key, aws_secret_access_key=secret_key)
    s3_resource = sess.resource("s3", endpoint_url=endpoint_url)

    return s3_resource


def plain_auth():
    """Sign S3 the auth manually"""
    config.load_config_file()
    access_key = os.environ.get("OBS_USER_ACCESS_KEY")
    secret_key = os.environ.get("OBS_USER_SECRET_KEY")
    endpoint = os.environ.get("OBS_USER_URL")

    # "eu-west-1" just fake region that we didn't use
    # it must be supplied otherwise key sign failed
    auth = AWS4Auth(access_key, secret_key, "eu-west-1", "s3")

    return auth, endpoint


def admin_client():
    """:return: CloudianApiClient object"""
    config.load_config_file()
    user = os.environ.get("OBS_ADMIN_USERNAME")
    passwd = os.environ.get("OBS_ADMIN_PASSWORD")
    endpoint = os.environ.get("OBS_ADMIN_URL")
    endpoint_url = f"http://{endpoint}"
    port = os.environ.get("OBS_ADMIN_PORT")
    client = CloudianAPIClient(url=endpoint_url, user=user, key=passwd, port=port)
    return client



import click
import sys

from obs.libs import auth
from obs.storage import bucket
from obs.storage import gmt


def warn_inexsit_config():
    msg = (
        f"Configuration file not available.\n"
        f"Consider running 'obs --configure' to create one"
    )
    click.secho(msg, fg="yellow", bold=True, err=True)


def get_resources():
    try:
        s3_resource = auth.resource()
        return s3_resource
    except Exception as exc:
        click.secho(str(exc), fg="yellow", bold=True, err=True)
        warn_inexsit_config()
        sys.exit(1)


def get_plain_auth():
    try:
        plain_auth = auth.plain_auth()
        return plain_auth
    except Exception as exc:
        click.secho(str(exc), fg="yellow", bold=True, err=True)
        warn_inexsit_config()
        sys.exit(1)


@click.group()
def storage():
    """Manage user storage."""


@storage.command("ls")
@click.argument("bucket_name", default="", required=False)
@click.option("-p", "prefix", default="", help="Prefix location")
def list(bucket_name, prefix):
    """List bucket or object."""
    s3_resource = get_resources()
    if bucket_name:
        bucket.get_objects(s3_resource, bucket_name=bucket_name, prefix=prefix)
    else:
        bucket.buckets(s3_resource)


@storage.command("rm")
@click.argument("target_name", nargs=-1)
def remove(target_name):
    """Remove bucket or object."""
    s3_resource = get_resources()
    if len(target_name) == 1:
        bucket.remove_bucket(s3_resource, bucket_name=target_name[0])
    if len(target_name) == 2:
        bucket_name, object_name = target_name
        bucket.remove_object(
            s3_resource, bucket_name=bucket_name, object_name=object_name
        )


@storage.command("mb")
@click.argument("bucket_name", default="")
@click.option(
    "--acl", "acl", default="private", required=False, help="Access Control List"
)
@click.option(
    "--policy-id",
    "policy_id",
    default="",
    required=False,
    help="Data distribution in bucket",
)
@click.option(
    "-r",
    "--random",
    "random_name",
    default=False,
    is_flag=True,
    help="Generate random name",
)
def make_bucket(bucket_name, acl, policy_id, random_name):
    """Create bucket."""
    plain_auth = get_plain_auth()
    bucket.create_bucket(
        auth=plain_auth,
        bucket_name=bucket_name,
        acl=acl,
        policy_id=policy_id,
        random_name=random_name,
    )


@storage.command("get")
@click.argument("bucket_name", default="")
@click.argument("object_name", default="")
def get_object(bucket_name, object_name):
    """Download object in bucket."""
    s3_resource = get_resources()
    bucket.download_object(
        s3_resource, bucket_name=bucket_name, object_name=object_name
    )


@storage.command("put")
@click.argument("bucket_name", default="")
@click.argument("path", default="")
@click.argument("object_name", default="", required=False)
@click.option(
    "--use-basename",
    "use_basename",
    default=False,
    is_flag=True,
    help="Use basename for object name",
)
def put_object(bucket_name, path, object_name, use_basename):
    """Upload object to bucket."""
    s3_resource = get_resources()
    bucket.upload_object(
        resource=s3_resource,
        bucket_name=bucket_name,
        path=path,
        object_name=object_name,
        use_basename=use_basename,
    )


@storage.command("cp")
@click.argument("src_bucket", default="")
@click.argument("dest_bucket", default="")
@click.argument("object_name", default="")
def copy_object(src_bucket, dest_bucket, object_name):
    """Copy object to other bucket."""
    s3_resource = get_resources()
    bucket.copy_object(
        s3_resource,
        src_bucket=src_bucket,
        dest_bucket=dest_bucket,
        object_name=object_name,
    )


@storage.command("mv")
@click.argument("src_bucket", default="")
@click.argument("dest_bucket", default="")
@click.argument("object_name", default="")
def move_object(src_bucket, dest_bucket, object_name):
    """Move object into other bucket."""
    s3_resource = get_resources()
    bucket.move_object(
        s3_resource,
        src_bucket=src_bucket,
        dest_bucket=dest_bucket,
        object_name=object_name,
    )


@storage.command("du")
@click.argument("bucket_name", default="", required=False)
def du(bucket_name):
    """Show disk or bucket usage."""
    s3_resource = get_resources()
    if bucket_name:
        bucket.bucket_usage(s3_resource, bucket_name=bucket_name)
    else:
        bucket.disk_usage(s3_resource)


@storage.command("info")
@click.argument("target_name", nargs=-1)
def info(target_name):
    """Display bucket or object info."""
    s3_resource = get_resources()
    plain_auth = get_plain_auth()
    if len(target_name) == 1:
        bucket_name = target_name[0]
        bucket.bucket_info(s3_resource, bucket_name=bucket_name, auth=plain_auth)
    if len(target_name) == 2:
        bucket_name, object_name = target_name
        bucket.object_info(
            s3_resource, bucket_name=bucket_name, object_name=object_name
        )


@storage.command("acl")
@click.argument("target_name", nargs=-1)
@click.argument("acl", default="private")
def set_acl(target_name, acl):
    """Set ACL for bucket or object."""
    s3_resource = get_resources()
    if len(target_name) == 1:
        acl_type = "bucket"
        bucket_name = target_name[0]
        bucket.set_acl(
            resource=s3_resource, bucket_name=bucket_name, acl=acl, acl_type=acl_type
        )

    if len(target_name) == 2:
        acl_type = "object"
        bucket_name, object_name = target_name
        bucket.set_acl(
            resource=s3_resource,
            bucket_name=bucket_name,
            object_name=object_name,
            acl=acl,
            acl_type=acl_type,
        )


@storage.command("presign")
@click.argument("target_name", nargs=-1)
@click.option("--expire", "expire", type=int, help="Set expiration time [default:3600]")
def url(target_name, expire):
    """Generate Url for bucket or object."""
    s3_resource = get_resources()
    if len(target_name) == 2:
        bucket_name, object_name = target_name
        bucket.generate_url(
            resource=s3_resource,
            bucket_name=bucket_name,
            object_name=object_name,
            expire=expire,
        )


@storage.command("mkdir")
@click.argument("target_name", nargs=-1)
def mkdir(target_name):
    """Create directory inside bucket"""
    s3_resource = get_resources()
    bucket_name, dir_name = target_name
    bucket.mkdir(resource=s3_resource, bucket_name=bucket_name, dir_name=dir_name)


@storage.command("gmt")
@click.option("--policy-id", "policy_id", is_flag=True, help="List Gmt Policy ID")
def gmt_cmd(policy_id):
    """Manage Cloudian extensions to S3."""
    if policy_id:
        gmt.show_policies()
